from distutils.core import setup
setup(name = 'fresnel',
    description = "light interaction with interfaces and multilayers" ,
    version = '0.1.1',
    author = "Angel Yanguas-Gil",
    author_email = "angel.yanguas@gmail.com",
    url = "http://ayanguasgil.net/",
    download_url = (
        "http://ayanguasgil.net/stuff/fresnel-0.1.1.tar.gz"),
    packages = ['fresnel'],
    classifiers = ["Development Status :: 2 - Pre-Alpha",
        "Environment :: Console",
        "Topic :: Scientific/Engineering",
        "Topic :: Utilities",
        "License :: OSI Approved :: BSD License" ]
)

